###########################################################################
#
#   Copyright (c) 2001 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.0 $

from ZPublisher.HTTPRequest import html_quote
html_quote_function = html_quote


class html_widget_base:
	"""Base class for HTML widgets."""

	type = 'html_widget_base'


	def render(self, *args, **kw):
		"""Returns an HTML presentation of the widget."""

		return apply(self.__call__, args, kw)


class checkbox(html_widget_base):
	"""Class for handling <input type="checkbox"> tags."""

	type = 'checkbox'


	def __init__(self, name='', value='', displayed_value='', checked=None):

		self.name = name
		self.value = value
		self.displayed_value = displayed_value or value
		self.checked = checked


	def __call__(self, html_quote=1):
		"""Returns an HTML presentation of the widget."""

		if html_quote:

			value = html_quote_function(self.value)
			displayed_value = html_quote_function(self.displayed_value)

		else:

			value = self.value
			displayed_value = self.displayed_value

		if self.checked:

			checked = " checked"

		else:

			checked = ""

		return """<input type="checkbox" name="%s" value="%s"%s>%s""" % \
				(self.name, value, checked, displayed_value)


class password(html_widget_base):
	"""Class for handling <input type="password"> tags."""

	type = 'password'


	def __init__(self, name='', *args, **kw):

		self.name = name


class hidden(html_widget_base):
	"""Class for handling <input type="hidden"> tags."""

	type = 'hidden'


	def __init__(self, name='', *args, **kw):

		self.name = name


class image(html_widget_base):
	"""Class for handling <input type="image"> tags."""

	type = 'image'


	def __init__(self, name='', *args, **kw):

		self.name = name


class reset(html_widget_base):
	"""Class for handling <input type="reset"> tags."""

	type = 'reset'


	def __init__(self, name='', *args, **kw):

		self.name = name


class submit(html_widget_base):
	"""Class for handling <input type="submit"> tags."""

	type = 'submit'


	def __init__(self, name='', *args, **kw):

		self.name = name


class file(html_widget_base):
	"""Class for handling <input type="file"> tags."""

	type = 'file'


	def __init__(self, name='', *args, **kw):

		self.name = name


class button(html_widget_base):
	"""Class for handling <input type="button"> tags."""

	type = 'button'


	def __init__(self, name='', value='', tabindex='',
			accesskey='', onfocus='', onblur=''):

		self.name = ''


class text(html_widget_base):
	"""Class for handling <input type="text"> tags."""

	type = 'text'


	def __init__(self, name=None, value=None, size=None,
			onclick=None, onchange=None):
		"""Initalizies the text widget."""

		self.name = name
		self.value = value
		self.size = size or 50

		self.onclick = onclick or None
		self.onchange = onchange or None


	def __call__(self, html_quote=1):
		"""Returns an HTML presentation of the widget."""

		if html_quote:

			value = html_quote_function(self.value)

		else:

			value = self.value

		return """<input type="text" name="%s" size="%s" value="%s" />""" % \
				(self.name, self.size, self.value)


class textarea(html_widget_base):
	"""Class for handling <textarea> tags."""

	type = 'textarea'


	def __init__(self, name='', value='', rows=None, cols=None):
		"""Initializes the textarea widget."""

		self.name = name
		self.value = value
		self.rows = rows or 10
		self.cols = cols or 50


	def __call__(self, html_quote=1):
		"""Returns an HTML presentation of the widget."""

		if html_quote:

			value = html_quote_function(self.value)

		else:

			value = self.value

		return """<textarea name="%s" rows="%s" cols="%s">%s</textarea>""" % \
				(self.name, self.rows, self.cols, value)


class select_option(html_widget_base):
	"""Class used for handling <option> tags within <select> tags."""

	type = 'select_option'


	def __init__(self, value, displayed_value=None,
			selected=None):
		"""Initializes the option."""

		self.value = value

		if displayed_value is None:

			self.displayed_value = self.value

		else:

			self.displayed_value = displayed_value

		self.selected = selected


	def select(self):
		"""Sets the selected flag to true."""

		self.selected = 1


	def unselect(self):
		"""Sets the selected flag to false."""

		self.selected = 0


	def __call__(self, html_quote=1):
		"""Returns an HTML presentation."""

		if self.selected:

			selected = " selected"

		else:

			selected = ""

		if html_quote:

			value = html_quote_function(self.value)
			displayed_value = html_quote_function(self.displayed_value)

		else:

			value = self.value
			displayed_value = self.displayed_value

		return """<option value="%s"%s>%s</option>""" % \
				(value, selected, displayed_value)


class radio(html_widget_base):
	"""Class for handling a single <input type="radio"> within a group of them."""

	type = 'radio'

	def __init__(self, name='', value='', displayed_value=None,
			checked=None):
		"""Initializes the option."""

		self.name = name
		self.value = value

		if displayed_value is None:

			self.displayed_value = value

		else:

			self.displayed_value = displayed_value

		self.checked = checked


	def check(self):
		"""Sets the checked flag to true."""

		self.checked = 1


	def uncheck(self):
		"""Sets the checked flag to false."""

		self.checked = 0


	def __call__(self, html_quote=1):
		"""Returns an HTML presentation."""

		if self.checked:

			checked = " checked"

		else:

			checked = ""

		if html_quote:

			value = html_quote_function(self.value)
			displayed_value = html_quote_function(self.displayed_value)

		else:

			value = self.value
			displayed_value = displayed_value

		return """<input type="radio" name="%s" value="%s"%s />%s""" % \
				(self.name, self.value, checked, self.displayed_value)


class radio_collection(html_widget_base):
	"""Class used for handling a selection of <input type="radio"> tags."""

	type = 'radio_collection'


	def __init__(self, name='', true='', false=''):
		"""Initialize the widget."""

		self.name = name
		self.true = true or 'Yes'
		self.false = false or 'No'
		self.options = []


	def add_option(self, value='', displayed_value='', checked=None):

		self.options.append(radio(name=self.name, value=value, displayed_value=displayed_value,
						checked=checked))


	def __call__(self, html_quote=1):
		"""Returns an HTML presentation of the widget."""

		html = ""

		for option in self.options:

			html = html + option.render(html_quote=html_quote)

		return html


class checkbox_collection(html_widget_base):
	"""Class used for handling sets of <input type="checkbox"> tags."""

	type = 'checkbox_collection'


	def __init__(self, name=''):
		"""Initializes the widget."""

		self.name = name
		self.options = []


	def add_option(self, value='', displayed_value='', checked=None):
		"""Adds an option to the collection."""

		self.options.append(checkbox(name=self.name, value=value,
				displayed_value=displayed_value, checked=checked))


	def __call__(self, html_quote=1):
		"""Returns an HTML presentation of the widget."""

		html = ""

		for option in self.options:

			html = html + option(html_quote=html_quote)

		return html


class select(html_widget_base):
	"""Class for handling <select> tags."""

	type = 'select'

	def __init__(self, name='', multiple=None, size=1):
		"""Initializes the widget."""

		self.name = name
		self.options = []
		self.multiple = multiple


	def add_option(self, option):
		"""Adds an option to the selection."""

		self.options.append(option)


	def __call__(self, html_quote=1):
		"""Returns an HTML presentation of the widget."""

		if self.multiple:

			multiple = " multiple"

		else:

			multiple = ""

		html = """<select name="%s"%s>""" % (self.name, multiple)

		for option in self.options:

			html = html + option(html_quote=html_quote)

		return html + "</select>"

